import { diag, trace, } from '@opentelemetry/api';
import { ATTR_SESSION_ID } from '@opentelemetry/semantic-conventions/incubating';
import { KEY_EMB_SESSION_REASON_ENDED, KEY_PREFIX_EMB_PROPERTIES, } from '../../constants/attributes.js';
import { EMB_STATES, EMB_TYPES, KEY_EMB_STATE, KEY_EMB_TYPE, } from '../../constants/index.js';
import { generateUUID, OTelPerformanceManager } from '../../utils/index.js';
var EmbraceSpanSessionManager = /** @class */ (function () {
    function EmbraceSpanSessionManager(_a) {
        var _b = _a === void 0 ? {} : _a, diagParam = _b.diag, perf = _b.perf;
        this._activeSessionId = null;
        this._activeSessionStartTime = null;
        this._sessionSpan = null;
        this._diag =
            diagParam !== null && diagParam !== void 0 ? diagParam : diag.createComponentLogger({
                namespace: 'EmbraceSpanSessionManager',
            });
        this._perf = perf !== null && perf !== void 0 ? perf : new OTelPerformanceManager();
    }
    // the external api doesn't include a reason, and if a users uses it to end a session, the reason will be 'user_ended'
    EmbraceSpanSessionManager.prototype.addBreadcrumb = function (name) {
        if (!this._sessionSpan) {
            this._diag.debug('trying to add breadcrumb to a session, but there is no session in progress. This is a no-op.');
            return;
        }
        this._sessionSpan.addEvent('emb-breadcrumb', {
            message: name,
        }, this._perf.getNowMillis());
    };
    EmbraceSpanSessionManager.prototype.addProperty = function (key, value) {
        if (!this._sessionSpan) {
            this._diag.debug('trying to add properties to a session, but there is no session in progress. This is a no-op.');
            return;
        }
        this._sessionSpan.setAttribute(KEY_PREFIX_EMB_PROPERTIES + key, value);
    };
    // note: don't use this internally, this is just for user facing APIs. Use this.endSessionSpanInternal instead.
    EmbraceSpanSessionManager.prototype.endSessionSpan = function () {
        this.endSessionSpanInternal('manual');
    };
    // endSessionSpanInternal is not part of the public API, but is used internally to end a session span adding a specific reason
    EmbraceSpanSessionManager.prototype.endSessionSpanInternal = function (reason) {
        if (!this._sessionSpan) {
            this._diag.debug('trying to end a session, but there is no session in progress. This is a no-op.');
            return;
        }
        this._sessionSpan.setAttribute(KEY_EMB_SESSION_REASON_ENDED, reason);
        this._sessionSpan.end();
        this._sessionSpan = null;
        this._activeSessionStartTime = null;
        this._activeSessionId = null;
    };
    EmbraceSpanSessionManager.prototype.getSessionId = function () {
        return this._activeSessionId;
    };
    EmbraceSpanSessionManager.prototype.getSessionSpan = function () {
        return this._sessionSpan;
    };
    EmbraceSpanSessionManager.prototype.getSessionStartTime = function () {
        return this._activeSessionStartTime;
    };
    EmbraceSpanSessionManager.prototype.startSessionSpan = function () {
        var _a;
        //if there was a session in progress already, finish it first.
        if (this._sessionSpan) {
            this.endSessionSpanInternal('manual');
        }
        var tracer = trace.getTracer('embrace-web-sdk-sessions');
        this._activeSessionId = generateUUID();
        this._activeSessionStartTime = this._perf.getNowHRTime();
        this._sessionSpan = tracer.startSpan('emb-session', {
            attributes: (_a = {},
                _a[KEY_EMB_TYPE] = EMB_TYPES.Session,
                _a[KEY_EMB_STATE] = EMB_STATES.Foreground,
                _a[ATTR_SESSION_ID] = this._activeSessionId,
                _a),
        });
    };
    return EmbraceSpanSessionManager;
}());
export { EmbraceSpanSessionManager };
//# sourceMappingURL=EmbraceSpanSessionManager.js.map