import { diag, trace, } from '@opentelemetry/api';
import { ATTR_SESSION_ID } from '@opentelemetry/semantic-conventions/incubating';
import { KEY_EMB_SESSION_REASON_ENDED, KEY_PREFIX_EMB_PROPERTIES, } from '../../constants/attributes.js';
import { EMB_STATES, EMB_TYPES, KEY_EMB_STATE, KEY_EMB_TYPE, } from '../../constants/index.js';
import { generateUUID, OTelPerformanceManager } from '../../utils/index.js';
export class EmbraceSpanSessionManager {
    constructor({ diag: diagParam, perf, } = {}) {
        this._activeSessionId = null;
        this._activeSessionStartTime = null;
        this._sessionSpan = null;
        this._diag =
            diagParam !== null && diagParam !== void 0 ? diagParam : diag.createComponentLogger({
                namespace: 'EmbraceSpanSessionManager',
            });
        this._perf = perf !== null && perf !== void 0 ? perf : new OTelPerformanceManager();
    }
    // the external api doesn't include a reason, and if a users uses it to end a session, the reason will be 'user_ended'
    addBreadcrumb(name) {
        if (!this._sessionSpan) {
            this._diag.debug('trying to add breadcrumb to a session, but there is no session in progress. This is a no-op.');
            return;
        }
        this._sessionSpan.addEvent('emb-breadcrumb', {
            message: name,
        }, this._perf.getNowMillis());
    }
    addProperty(key, value) {
        if (!this._sessionSpan) {
            this._diag.debug('trying to add properties to a session, but there is no session in progress. This is a no-op.');
            return;
        }
        this._sessionSpan.setAttribute(KEY_PREFIX_EMB_PROPERTIES + key, value);
    }
    // note: don't use this internally, this is just for user facing APIs. Use this.endSessionSpanInternal instead.
    endSessionSpan() {
        this.endSessionSpanInternal('manual');
    }
    // endSessionSpanInternal is not part of the public API, but is used internally to end a session span adding a specific reason
    endSessionSpanInternal(reason) {
        if (!this._sessionSpan) {
            this._diag.debug('trying to end a session, but there is no session in progress. This is a no-op.');
            return;
        }
        this._sessionSpan.setAttribute(KEY_EMB_SESSION_REASON_ENDED, reason);
        this._sessionSpan.end();
        this._sessionSpan = null;
        this._activeSessionStartTime = null;
        this._activeSessionId = null;
    }
    getSessionId() {
        return this._activeSessionId;
    }
    getSessionSpan() {
        return this._sessionSpan;
    }
    getSessionStartTime() {
        return this._activeSessionStartTime;
    }
    startSessionSpan() {
        //if there was a session in progress already, finish it first.
        if (this._sessionSpan) {
            this.endSessionSpanInternal('manual');
        }
        const tracer = trace.getTracer('embrace-web-sdk-sessions');
        this._activeSessionId = generateUUID();
        this._activeSessionStartTime = this._perf.getNowHRTime();
        this._sessionSpan = tracer.startSpan('emb-session', {
            attributes: {
                [KEY_EMB_TYPE]: EMB_TYPES.Session,
                [KEY_EMB_STATE]: EMB_STATES.Foreground,
                [ATTR_SESSION_ID]: this._activeSessionId,
            },
        });
    }
}
//# sourceMappingURL=EmbraceSpanSessionManager.js.map