"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EmbraceSpanSessionManager = void 0;
const api_1 = require("@opentelemetry/api");
const incubating_1 = require("@opentelemetry/semantic-conventions/incubating");
const attributes_js_1 = require("../../constants/attributes.js");
const index_js_1 = require("../../constants/index.js");
const index_js_2 = require("../../utils/index.js");
class EmbraceSpanSessionManager {
    constructor({ diag: diagParam, perf, } = {}) {
        this._activeSessionId = null;
        this._activeSessionStartTime = null;
        this._sessionSpan = null;
        this._diag =
            diagParam !== null && diagParam !== void 0 ? diagParam : api_1.diag.createComponentLogger({
                namespace: 'EmbraceSpanSessionManager',
            });
        this._perf = perf !== null && perf !== void 0 ? perf : new index_js_2.OTelPerformanceManager();
    }
    // the external api doesn't include a reason, and if a users uses it to end a session, the reason will be 'user_ended'
    addBreadcrumb(name) {
        if (!this._sessionSpan) {
            this._diag.debug('trying to add breadcrumb to a session, but there is no session in progress. This is a no-op.');
            return;
        }
        this._sessionSpan.addEvent('emb-breadcrumb', {
            message: name,
        }, this._perf.getNowMillis());
    }
    addProperty(key, value) {
        if (!this._sessionSpan) {
            this._diag.debug('trying to add properties to a session, but there is no session in progress. This is a no-op.');
            return;
        }
        this._sessionSpan.setAttribute(attributes_js_1.KEY_PREFIX_EMB_PROPERTIES + key, value);
    }
    // note: don't use this internally, this is just for user facing APIs. Use this.endSessionSpanInternal instead.
    endSessionSpan() {
        this.endSessionSpanInternal('manual');
    }
    // endSessionSpanInternal is not part of the public API, but is used internally to end a session span adding a specific reason
    endSessionSpanInternal(reason) {
        if (!this._sessionSpan) {
            this._diag.debug('trying to end a session, but there is no session in progress. This is a no-op.');
            return;
        }
        this._sessionSpan.setAttribute(attributes_js_1.KEY_EMB_SESSION_REASON_ENDED, reason);
        this._sessionSpan.end();
        this._sessionSpan = null;
        this._activeSessionStartTime = null;
        this._activeSessionId = null;
    }
    getSessionId() {
        return this._activeSessionId;
    }
    getSessionSpan() {
        return this._sessionSpan;
    }
    getSessionStartTime() {
        return this._activeSessionStartTime;
    }
    startSessionSpan() {
        //if there was a session in progress already, finish it first.
        if (this._sessionSpan) {
            this.endSessionSpanInternal('manual');
        }
        const tracer = api_1.trace.getTracer('embrace-web-sdk-sessions');
        this._activeSessionId = (0, index_js_2.generateUUID)();
        this._activeSessionStartTime = this._perf.getNowHRTime();
        this._sessionSpan = tracer.startSpan('emb-session', {
            attributes: {
                [index_js_1.KEY_EMB_TYPE]: index_js_1.EMB_TYPES.Session,
                [index_js_1.KEY_EMB_STATE]: index_js_1.EMB_STATES.Foreground,
                [incubating_1.ATTR_SESSION_ID]: this._activeSessionId,
            },
        });
    }
}
exports.EmbraceSpanSessionManager = EmbraceSpanSessionManager;
//# sourceMappingURL=EmbraceSpanSessionManager.js.map