"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initSDK = void 0;
const api_1 = require("@opentelemetry/api");
const api_logs_1 = require("@opentelemetry/api-logs");
const instrumentation_1 = require("@opentelemetry/instrumentation");
const resources_1 = require("@opentelemetry/resources");
const sdk_logs_1 = require("@opentelemetry/sdk-logs");
const sdk_trace_web_1 = require("@opentelemetry/sdk-trace-web");
const index_js_1 = require("../api-sessions/index.js");
const index_js_2 = require("../api-users/index.js");
const index_js_3 = require("../exporters/index.js");
const index_js_4 = require("../managers/index.js");
const index_js_5 = require("../processors/index.js");
const index_js_6 = require("../resources/index.js");
const utils_js_1 = require("./utils.js");
const setupDefaultInstrumentations_js_1 = require("./setupDefaultInstrumentations.js");
const web_common_1 = require("@opentelemetry/web-common");
const index_js_7 = require("../api-logs/index.js");
const index_js_8 = require("../api-traces/index.js");
const registry_js_1 = require("./registry.js");
const initSDK = ({ appID, appVersion, resource = resources_1.Resource.empty(), spanExporters = [], logExporters = [], spanProcessors = [], propagator = null, defaultInstrumentationConfig, instrumentations = [], contextManager = null, logProcessors = [], logLevel = api_1.DiagLogLevel.ERROR, diagLogger = api_1.diag.createComponentLogger({
    namespace: 'embrace-sdk',
}), } = { appID: '' }) => {
    var _a;
    try {
        const existingSDK = registry_js_1.registry.registered();
        if (existingSDK !== null) {
            diagLogger.warn('SDK has already been successfully initialized, skipping this invocation of initSDK');
            return existingSDK;
        }
        api_1.diag.setLogger(new api_1.DiagConsoleLogger(), {
            logLevel,
        });
        const resourceWithWebSDKAttributes = resource.merge((0, index_js_6.getWebSDKResource)(appVersion));
        const sendingToEmbrace = !!appID && (0, utils_js_1.isValidAppID)(appID);
        if (!sendingToEmbrace && !logExporters.length && !spanExporters.length) {
            throw new Error('when the embrace appID is omitted then at least one logExporter or spanExporter must be set');
        }
        const userManager = setupUser();
        const enduserPseudoID = (_a = userManager.getUser()) === null || _a === void 0 ? void 0 : _a[index_js_2.KEY_ENDUSER_PSEUDO_ID];
        if (sendingToEmbrace && !enduserPseudoID) {
            throw new Error('userID is required when using Embrace exporter');
        }
        const spanSessionManager = setupSession();
        const tracerProvider = setupTraces({
            sendingToEmbrace,
            appID,
            enduserPseudoID,
            resource: resourceWithWebSDKAttributes,
            spanSessionManager,
            spanExporters,
            spanProcessors,
            propagator,
            contextManager,
        });
        const loggerProvider = setupLogs({
            sendingToEmbrace,
            appID,
            enduserPseudoID,
            resource: resourceWithWebSDKAttributes,
            logExporters,
            logProcessors,
            spanSessionManager,
        });
        // NOTE: we require setupInstrumentation to run the last, after setupLogs and setupTraces. This is how OTel works wrt
        // the dependencies between instrumentations and global providers. We need the providers for tracers, and logs to be
        // setup before we enable instrumentations.
        (0, instrumentation_1.registerInstrumentations)({
            instrumentations: [
                ...instrumentations,
                (0, setupDefaultInstrumentations_js_1.setupDefaultInstrumentations)(defaultInstrumentationConfig),
            ],
        });
        diagLogger.info('successfully initialized the SDK');
        const sdkControl = {
            flush: async () => {
                await tracerProvider.forceFlush();
                await loggerProvider.forceFlush();
            },
        };
        registry_js_1.registry.register(sdkControl);
        return sdkControl;
    }
    catch (e) {
        const message = e instanceof Error ? e.message : 'Unknown error.';
        diagLogger.error(`failed to initialize the SDK: ${message}`);
        return false;
    }
};
exports.initSDK = initSDK;
const setupUser = () => {
    const embraceUserManager = new index_js_4.EmbraceUserManager();
    index_js_2.user.setGlobalUserManager(embraceUserManager);
    return embraceUserManager;
};
const setupSession = () => {
    const embraceSpanSessionManager = new index_js_4.EmbraceSpanSessionManager();
    index_js_1.session.setGlobalSessionManager(embraceSpanSessionManager);
    return embraceSpanSessionManager;
};
const setupTraces = ({ sendingToEmbrace, appID, enduserPseudoID, resource, spanSessionManager, spanExporters, spanProcessors = [], propagator = null, contextManager = null, }) => {
    const embraceTraceManager = new index_js_4.EmbraceTraceManager();
    index_js_8.trace.setGlobalTraceManager(embraceTraceManager);
    const finalSpanProcessors = [
        ...spanProcessors,
        (0, web_common_1.createSessionSpanProcessor)(spanSessionManager),
        new index_js_5.EmbraceNetworkSpanProcessor(),
    ];
    spanExporters === null || spanExporters === void 0 ? void 0 : spanExporters.forEach(exporter => {
        finalSpanProcessors.push(new sdk_trace_web_1.BatchSpanProcessor(exporter));
    });
    if (sendingToEmbrace && appID && enduserPseudoID) {
        finalSpanProcessors.push(new index_js_5.EmbraceSessionBatchedSpanProcessor({
            exporter: new index_js_3.EmbraceTraceExporter({
                appID,
                userID: enduserPseudoID,
            }),
        }));
    }
    const tracerProvider = new sdk_trace_web_1.WebTracerProvider({
        resource,
        spanProcessors: finalSpanProcessors,
    });
    tracerProvider.register({
        contextManager,
        propagator,
    });
    return tracerProvider;
};
const setupLogs = ({ sendingToEmbrace, appID, enduserPseudoID, resource, logExporters, logProcessors, spanSessionManager, }) => {
    const embraceLogManager = new index_js_4.EmbraceLogManager();
    index_js_7.log.setGlobalLogManager(embraceLogManager);
    const loggerProvider = new sdk_logs_1.LoggerProvider({
        resource,
    });
    const finalLogProcessors = [
        ...logProcessors,
        new index_js_5.IdentifiableSessionLogRecordProcessor({
            spanSessionManager,
        }),
        new index_js_5.EmbTypeLogRecordProcessor(),
    ];
    logExporters === null || logExporters === void 0 ? void 0 : logExporters.forEach(exporter => {
        finalLogProcessors.push(new sdk_logs_1.BatchLogRecordProcessor(exporter));
    });
    if (sendingToEmbrace && appID && enduserPseudoID) {
        finalLogProcessors.push(new sdk_logs_1.BatchLogRecordProcessor(new index_js_3.EmbraceLogExporter({
            appID,
            userID: enduserPseudoID,
        })));
    }
    for (const logProcessor of finalLogProcessors) {
        loggerProvider.addLogRecordProcessor(logProcessor);
    }
    api_logs_1.logs.setGlobalLoggerProvider(loggerProvider);
    return loggerProvider;
};
//# sourceMappingURL=initSDK.js.map